<template>
  <section class="grid">
    <header class="button-header" v-if="false">
      Reload
      Pagination
    </header>
    <header class="row-header" :style="{ 'padding-right' : scrollBarWidth + 'px' }" style="padding-left: 3px">
      <header class="row no-gutters">
        <section class="column-header" v-for="column in columns" :class="{ ['col-' + ( column.span || 1 )] : true }">
          {{column.name}}
        </section>
      </header>
    </header>
    <section class="data-row" v-if="compiledData.length > 0">
      <section v-for="row in compiledData" @click="SelectRow( row )" class="row no-gutters" :class="{ 'selected' : selectedRow === row }">
        <section class="column" v-for="column in columns" :class="{ ['col-' + ( column.span || 1 ) ] : true }"  >
          {{column.view ? column.view( row[column.field] ) : row[column.field] }}
        </section>
      </section>
    </section>
    <section class="no-rows" v-else align="center">
      {{ norows }}
    </section>
  </section>
</template>

<script>

  import Vue from 'vue';

  export default Vue.component( 'grid', {

    props : {
      dataSource : {
        type : String,
        default : null
      },
      columns : {
        type : Array,
        default : []
      },
      length : {
        type : Number,
        default : null
      },
      limit : {
        type : Number,
        default : 100
      },
      norows: {
        type: String,
        default: "No rows of data found."
      },
      orderBy: {
        type: String,
        default: null
      },
      orderByOrder: {
        type: String,
        default: 'ASC'
      },
      parameters : {
        type: Object,
        default: () => {
          return {};
        }
      }
    },

    created(){
      this.Reload();
      this.scrollBarWidth = this.ScrollbarWidth();
    },
    methods : {
      Reload(){
        if(this.dataSource !== null){
          app.api.Get( this.dataSource, Object.assign(this.parameters, {limit: this.limit})).then( ( response ) => {
          if(this.orderBy){
            var orderBy = this.orderBy;
            if(this.orderByOrder === 'DESC'){
              response = response.sort(function (a, b) {
                if (b[orderBy] < a[orderBy])
                  return -1;
                if (b[orderBy] > a[orderBy])
                  return 1;
                return 0;
              });
            }
            else{
              response = response.sort(function (a, b) {
                if (a[orderBy] < b[orderBy])
                  return -1;
                if (a[orderBy] > b[orderBy])
                  return 1;
                return 0;
              });
            }
          }
            this.compiledData = response;
          } );
        }
      },
      DeselectRow( emit = true ){
        this.selectedRow = null;
        if( emit ){
          this.$emit( 'rowSelected', this.selectedRow );
        }

      },
      SelectRow( row ){
        if( this.selectedRow === row ){
          this.selectedRow = null;
        }
        else{
          this.selectedRow = row;
        }

        this.$emit( 'rowSelected', this.selectedRow );
      },
      ScrollbarWidth() {
        let outer = document.createElement("div");
        outer.style.visibility = "hidden";
        outer.style.width = "100px";
        outer.style.msOverflowStyle = "scrollbar"; // needed for WinJS apps

        document.body.appendChild(outer);

        let widthNoScroll = outer.offsetWidth;
        // force scrollbars
        outer.style.overflow = "scroll";

        // add innerdiv
        let inner = document.createElement("div");
        inner.style.width = "100%";
        outer.appendChild(inner);        

        let widthWithScroll = inner.offsetWidth;

        // remove divs
        outer.parentNode.removeChild(outer);

        return widthNoScroll - widthWithScroll;
      }
    },

    data(){
      return {
        compiledData : [],
        scrollBarWidth : 0,
        selectedRow : null
      }
    }

  } );

</script>

<style lang="less" scoped>

  .grid{
    header.button-header{
      margin-bottom: 10px;
    }

    header.row-header {
      background-color: #343a40;
      border-radius: 5px 5px 0px 0px;
      border-right: solid 2px black;
      border-bottom: solid 2px black;
      border-top: solid 2px black;

      section.column-header {
        color: white;
        padding : 5px;
        border-right: solid 1px black;

        &:last-child{
          border: none;
        }
      }
    }

    .data-row{
      border: solid 1px black;
      border-top: none;
      max-height: 500px;
      overflow-y: scroll;
      overflow-x: hidden;
      margin-top: 2px;
      border-radius: 0px 0px 0px 5px;
      .row{

        
        border-bottom: solid 1px black;

        &:focus{
          outline: 2px solid #ffc107;
        }

        &:last-child{
          border-radius: 0px 0px 5px 5px;
          border-bottom:none;

          .column:first-child{
            border-radius: 0px 0px 0px 5px;
          }
        }

        &:nth-child( even ){
          background-color: #e9ecef;
        }

        &:nth-child( odd ){
          background-color: white;
        }

        &.selected{
          background-color: rgb( 48, 124, 255 );
          color: white;

          .btn-outline-primary{
            border-color: white;
            background-color: #343a40;
            color: white;
          }

          .btn-outline-primary:hover{
            border-color: #ffc107;
            background-color: #343a40;
            color: #ffc107;
          }
        }

        .column{
          //display: flex;

          &:focus{
            .columnContents{
              outline: 2px solid #ffc107;
            }
          }

          &:first-child{
            border-left: none;
          }

          .editInput{
            width: 100%;
            height: 34px;
            border: none;
            padding: 5px;
            &:focus{
              outline: 2px solid #ffc107;
            }
          }

          select{
            width: 100%;
            height: 34px;
            border: none;
            padding: 5px;
            &:focus{
              outline: 2px solid #ffc107;
            }
          }

          position: relative;
          border-left: solid 1px black;
          padding: 5px;
          padding-right: 10px;
        }
      }
    }

    .no-rows{
      border: solid 1px black;
      border-top: none;
      padding: 5px;
    }
  }

</style>